#ifndef MURMURHASH2NEUTRAL_HPP_
#define MURMURHASH2NEUTRAL_HPP_

//-----------------------------------------------------------------------------
// MurmurHashNeutral2, by Austin Appleby

// Same as MurmurHash2, but endian- and alignment-neutral.
// Half the speed though, alas.

//unsigned int MurmurHashNeutral2 ( const void * key, int len, unsigned int seed )
//{
//	const unsigned int m = 0x5bd1e995;
//	const int r = 24;
//
//	unsigned int h = seed ^ len;
//
//	const unsigned char * data = (const unsigned char *)key;
//
//	while(len >= 4)
//	{
//		unsigned int k;
//
//		k  = data[0];
//		k |= data[1] << 8;
//		k |= data[2] << 16;
//		k |= data[3] << 24;
//
//		k *= m;
//		k ^= k >> r;
//		k *= m;
//
//		h *= m;
//		h ^= k;
//
//		data += 4;
//		len -= 4;
//	}
//
//	switch(len)
//	{
//	case 3: h ^= data[2] << 16;
//	case 2: h ^= data[1] << 8;
//	case 1: h ^= data[0];
//	        h *= m;
//	};
//
//	h ^= h >> 13;
//	h *= m;
//	h ^= h >> 15;
//
//	return h;
//}

inline unsigned int MurmurHashNeutral2 ( const std::vector<int>& key, unsigned int seed )
{
	const unsigned int m = 0x5bd1e995;
	const int r = 24;
	int len = key.size();
	unsigned int pos = 0;

	unsigned int h = seed ^ len;

	//const unsigned char * data = (const unsigned char *)key;
	while(len >= 4)
	{
		unsigned int k;

		k  = key[pos];
		k |= key[pos+1] << 8;
		k |= key[pos+2] << 16;
		k |= key[pos+3] << 24;

		k *= m;
		k ^= k >> r;
		k *= m;

		h *= m;
		h ^= k;

		pos += 4;
		len -= 4;
	}

	switch(len)
	{
	case 3: h ^= key[pos+2] << 16;
	case 2: h ^= key[pos+1] << 8;
	case 1: h ^= key[pos];
	        h *= m;
	};

	h ^= h >> 13;
	h *= m;
	h ^= h >> 15;

	return h;
}

#endif /* MURMURHASH2NEUTRAL_HPP_ */
